@@ -1,13 +1,12 @@
 # -*- coding: utf-8 -*-
 
+import importlib.metadata
 import logging
 
-import pkg_resources
-
 logging.basicConfig(
     level=logging.INFO, format="%(asctime)s:%(levelname)s:%(name)s:%(message)s"
 )
 
 
 def get_bugbug_version():
-    return pkg_resources.get_distribution("bugbug").version
+    return importlib.metadata.version("bugbug")
