@@ -4,12 +4,12 @@
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import logging
-from typing import Callable, Iterator, List, NewType, Tuple
+from typing import Callable, Iterator, NewType
 
 from ratelimit import limits, sleep_and_retry
 
 from bugbug import db
-from bugbug.utils import get_secret, get_session
+from bugbug.utils import get_secret, get_session, get_user_agent
 
 logger = logging.getLogger(__name__)
 
@@ -58,18 +58,24 @@ class Github:
 
     def fetch_events(self, events_url: str) -> list:
         self.api_limit()
-        logger.info(f"Fetching {events_url}")
-        headers = {"Authorization": "token {}".format(self.get_token())}
+        logger.info("Fetching %s", events_url)
+        headers = {
+            "Authorization": "token {}".format(self.get_token()),
+            "User-Agent": get_user_agent(),
+        }
         response = get_session("github").get(events_url, headers=headers)
         response.raise_for_status()
         events_raw = response.json()
         return events_raw
 
     def fetch_issues(
-        self, url: str, retrieve_events: bool, params: dict = None
-    ) -> Tuple[List[IssueDict], dict]:
+        self, url: str, retrieve_events: bool, params: dict | None = None
+    ) -> tuple[list[IssueDict], dict]:
         self.api_limit()
-        headers = {"Authorization": "token {}".format(self.get_token())}
+        headers = {
+            "Authorization": "token {}".format(self.get_token()),
+            "User-Agent": get_user_agent(),
+        }
         response = get_session("github").get(url, params=params, headers=headers)
         response.raise_for_status()
         data = response.json()
@@ -78,7 +84,7 @@ class Github:
         if isinstance(data, dict):
             data = [data]
 
-        logger.info(f"Fetching {url}")
+        logger.info("Fetching %s", url)
 
         if retrieve_events:
             for item in data:
@@ -93,7 +99,7 @@ class Github:
         count = sum(1 for _ in issues)
         return int(count / PER_PAGE) + 1
 
-    def fetch_issues_updated_since_timestamp(self, since: str) -> List[IssueDict]:
+    def fetch_issues_updated_since_timestamp(self, since: str) -> list[IssueDict]:
         # Fetches changed and new issues since a specified timestamp
         url = "https://api.github.com/repos/{}/{}/issues".format(self.owner, self.repo)
 
