@@ -3,6 +3,7 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
+import logging
 from collections import Counter
 
 import xgboost
@@ -24,6 +25,9 @@ ADDRESSES_TO_EXCLUDE = [
     "nobody@t4b.me",
 ]
 
+logging.basicConfig(level=logging.INFO)
+logger = logging.getLogger(__name__)
+
 
 class AssigneeModel(BugModel):
     def __init__(self, lemmatization=False):
@@ -33,17 +37,17 @@ class AssigneeModel(BugModel):
         self.calculate_importance = False
 
         feature_extractors = [
-            bug_features.has_str(),
-            bug_features.severity(),
-            bug_features.keywords(),
-            bug_features.is_coverity_issue(),
-            bug_features.has_crash_signature(),
-            bug_features.has_url(),
-            bug_features.has_w3c_url(),
-            bug_features.has_github_url(),
-            bug_features.whiteboard(),
-            bug_features.patches(),
-            bug_features.landings(),
+            bug_features.HasSTR(),
+            bug_features.Severity(),
+            bug_features.Keywords(),
+            bug_features.IsCoverityIssue(),
+            bug_features.HasCrashSignature(),
+            bug_features.HasURL(),
+            bug_features.HasW3CURL(),
+            bug_features.HasGithubURL(),
+            bug_features.Whiteboard(),
+            bug_features.Patches(),
+            bug_features.Landings(),
         ]
 
         cleanup_functions = [
@@ -63,6 +67,11 @@ class AssigneeModel(BugModel):
                         rollback_when=self.rollback,
                     ),
                 ),
+            ]
+        )
+
+        self.clf = Pipeline(
+            [
                 (
                     "union",
                     ColumnTransformer(
@@ -77,12 +86,13 @@ class AssigneeModel(BugModel):
                         ]
                     ),
                 ),
+                (
+                    "estimator",
+                    xgboost.XGBClassifier(n_jobs=utils.get_physical_cpu_count()),
+                ),
             ]
         )
 
-        self.clf = xgboost.XGBClassifier(n_jobs=utils.get_physical_cpu_count())
-        self.clf.set_params(predictor="cpu_predictor")
-
     def get_labels(self):
         classes = {}
 
@@ -100,9 +110,9 @@ class AssigneeModel(BugModel):
             if count > MINIMUM_ASSIGNMENTS
         )
 
-        print(f"{len(top_assignees)} assignees")
+        logger.info("%d assignees", len(top_assignees))
         for assignee, count in assignee_counts:
-            print(f"{assignee}: {count}")
+            logger.info("%s: %d", assignee, count)
 
         classes = {
             bug_id: assignee
@@ -113,7 +123,7 @@ class AssigneeModel(BugModel):
         return classes, set(classes.values())
 
     def get_feature_names(self):
-        return self.extraction_pipeline.named_steps["union"].get_feature_names()
+        return self.clf.named_steps["union"].get_feature_names_out()
 
     def rollback(self, change):
         return change["field_name"].startswith("assigned_to")
