@@ -3,6 +3,7 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
+import logging
 from collections import Counter
 from datetime import datetime, timezone
 
@@ -17,12 +18,16 @@ from bugbug import bug_features, bugzilla, feature_cleanup, utils
 from bugbug.bugzilla import get_product_component_count
 from bugbug.model import BugModel
 
+logging.basicConfig(level=logging.INFO)
+logger = logging.getLogger(__name__)
+
 
 class ComponentModel(BugModel):
     PRODUCTS = {
         "Core",
         "External Software Affecting Firefox",
         "DevTools",
+        "Firefox for Android",
         "Firefox",
         "Toolkit",
         "WebExtensions",
@@ -48,6 +53,7 @@ class ComponentModel(BugModel):
         "External Software Affecting Firefox",
         "WebExtensions",
         "Firefox Build System",
+        "Firefox for Android",
     ]
 
     CONFLATED_COMPONENTS_MAPPING = {
@@ -58,6 +64,7 @@ class ComponentModel(BugModel):
         "External Software Affecting Firefox": "External Software Affecting Firefox::Other",
         "WebExtensions": "WebExtensions::Untriaged",
         "Firefox Build System": "Firefox Build System::General",
+        "Firefox for Android": "Firefox for Android::General",
     }
 
     def __init__(self, lemmatization=False):
@@ -67,17 +74,16 @@ class ComponentModel(BugModel):
         self.calculate_importance = False
 
         feature_extractors = [
-            bug_features.has_str(),
-            bug_features.severity(),
-            bug_features.keywords(),
-            bug_features.is_coverity_issue(),
-            bug_features.has_crash_signature(),
-            bug_features.has_url(),
-            bug_features.has_w3c_url(),
-            bug_features.has_github_url(),
-            bug_features.whiteboard(),
-            bug_features.patches(),
-            bug_features.landings(),
+            bug_features.HasSTR(),
+            bug_features.Severity(),
+            bug_features.Keywords(),
+            bug_features.HasCrashSignature(),
+            bug_features.HasURL(),
+            bug_features.HasW3CURL(),
+            bug_features.HasGithubURL(),
+            bug_features.Whiteboard(),
+            bug_features.Patches(),
+            bug_features.Landings(),
         ]
 
         cleanup_functions = [
@@ -94,6 +100,11 @@ class ComponentModel(BugModel):
                         feature_extractors, cleanup_functions, rollback=True
                     ),
                 ),
+            ]
+        )
+
+        self.clf = Pipeline(
+            [
                 (
                     "union",
                     ColumnTransformer(
@@ -108,17 +119,21 @@ class ComponentModel(BugModel):
                         ]
                     ),
                 ),
+                (
+                    "estimator",
+                    xgboost.XGBClassifier(n_jobs=utils.get_physical_cpu_count()),
+                ),
             ]
         )
 
-        self.clf = xgboost.XGBClassifier(n_jobs=utils.get_physical_cpu_count())
-        self.clf.set_params(predictor="cpu_predictor")
-
         self.CONFLATED_COMPONENTS_INVERSE_MAPPING = {
             v: k for k, v in self.CONFLATED_COMPONENTS_MAPPING.items()
         }
 
     def filter_component(self, product, component):
+        if product == "Firefox for Android" or product == "GeckoView":
+            return "Firefox for Android"
+
         full_comp = f"{product}::{component}"
 
         if full_comp in self.CONFLATED_COMPONENTS_INVERSE_MAPPING:
@@ -164,9 +179,9 @@ class ComponentModel(BugModel):
         component_counts = Counter(classes.values()).most_common()
         top_components = set(component for component, count in component_counts)
 
-        print(f"{len(top_components)} components")
+        logger.info("%d components", len(top_components))
         for component, count in component_counts:
-            print(f"{component}: {count}")
+            logger.info("%s: %d", component, count)
 
         # Assert there is at least one bug for each conflated component.
         for conflated_component in self.CONFLATED_COMPONENTS:
@@ -199,31 +214,36 @@ class ComponentModel(BugModel):
 
     def is_meaningful(self, product, component):
         return (
-            product in self.PRODUCTS and component not in ["General", "Untriaged"]
+            product in self.PRODUCTS
+            and component not in ["General", "Untriaged", "Foxfooding"]
         ) or (
             product in self.PRODUCT_COMPONENTS
             and component in self.PRODUCT_COMPONENTS[product]
         )
 
     def get_meaningful_product_components(self, full_comp_tuples, threshold_ratio=100):
-        """From the given full_comp_tuples iterable of (product, component)
-        tuples, returns the set of tuples which have at least 1% of the most
-        common tuple
-        """
+        """Filter out components which does not have more than 1% of the most common component.
 
+        Returns:
+            a set of tuples which have at least 1% of the most common tuple
+        """
         product_component_counts = Counter(full_comp_tuples).most_common()
 
         max_count = product_component_counts[0][1]
         threshold = max_count / threshold_ratio
 
+        active_product_components = bugzilla.get_active_product_components(
+            list(self.PRODUCTS) + list(self.PRODUCT_COMPONENTS)
+        )
+
         return set(
             product_component
             for product_component, count in product_component_counts
-            if count > threshold
+            if count > threshold and product_component in active_product_components
         )
 
     def get_feature_names(self):
-        return self.extraction_pipeline.named_steps["union"].get_feature_names()
+        return self.clf.named_steps["union"].get_feature_names_out()
 
     def check(self):
         success = super().check()
@@ -241,14 +261,18 @@ class ComponentModel(BugModel):
             full_comp = f"{product}::{component}"
 
             if full_comp not in bugs_number.keys():
-                print(
-                    f"Component {component!r} of product {product!r} doesn't exists, failure"
+                logger.warning(
+                    "Component %r of product %r doesn't exists, failure",
+                    component,
+                    product,
                 )
                 success = False
 
             elif bugs_number[full_comp] <= 0:
-                print(
-                    f"Component {component!r} of product {product!r} have 0 bugs or less in it, failure"
+                logger.warning(
+                    "Component %r of product %r have 0 bugs or less in it, failure",
+                    component,
+                    product,
                 )
                 success = False
 
@@ -257,7 +281,6 @@ class ComponentModel(BugModel):
         # than 0 bugs
 
         for conflated_component in self.CONFLATED_COMPONENTS:
-
             matching_components = [
                 full_comp
                 for full_comp in bugs_number
@@ -265,7 +288,7 @@ class ComponentModel(BugModel):
             ]
 
             if not matching_components:
-                print(f"{conflated_component} doesn't match any component")
+                logger.warning("%s doesn't match any component", conflated_component)
                 success = False
                 continue
 
@@ -276,8 +299,9 @@ class ComponentModel(BugModel):
             ]
 
             if not matching_components_values:
-                print(
-                    f"{conflated_component} should match at least one component with more than 0 bugs"
+                logger.warning(
+                    "%s should match at least one component with more than 0 bugs",
+                    conflated_component,
                 )
                 success = False
 
@@ -285,15 +309,16 @@ class ComponentModel(BugModel):
         # still exist as components and have more than 0 bugs
 
         for full_comp in self.CONFLATED_COMPONENTS_MAPPING.values():
-
             if full_comp not in bugs_number:
-                print(
-                    f"{full_comp} from conflated component mapping doesn't exists, failure"
+                logger.warning(
+                    "%s from conflated component mapping doesn't exists, failure",
+                    full_comp,
                 )
                 success = False
             elif bugs_number[full_comp] <= 0:
-                print(
-                    f"{full_comp} from conflated component mapping have less than 1 bug, failure"
+                logger.warning(
+                    "%s from conflated component mapping have less than 1 bug, failure",
+                    full_comp,
                 )
                 success = False
 
@@ -301,7 +326,6 @@ class ComponentModel(BugModel):
         # exist as components or are in CONFLATED_COMPONENTS_MAPPING
 
         for conflated_component in self.CONFLATED_COMPONENTS:
-
             in_mapping = conflated_component in self.CONFLATED_COMPONENTS_MAPPING
 
             matching_components = [
@@ -311,7 +335,7 @@ class ComponentModel(BugModel):
             ]
 
             if not (matching_components or in_mapping):
-                print(f"It should be possible to map {conflated_component}")
+                logger.warning("It should be possible to map %s", conflated_component)
                 success = False
                 continue
 
@@ -338,15 +362,16 @@ class ComponentModel(BugModel):
         if not meaningful_product_components.issubset(
             self.meaningful_product_components
         ):
-            print("Meaningful product components mismatch")
+            logger.warning("Meaningful product components mismatch")
 
             new_meaningful_product_components = (
                 meaningful_product_components.difference(
                     self.meaningful_product_components
                 )
             )
-            print(
-                f"New meaningful product components {new_meaningful_product_components!r}"
+            logger.info(
+                "New meaningful product components %r",
+                new_meaningful_product_components,
             )
 
             success = False
