@@ -6,11 +6,15 @@
 import sys
 from collections import defaultdict
 from functools import lru_cache
+from logging import INFO, basicConfig, getLogger
 
 import numpy as np
 from sklearn.base import BaseEstimator, TransformerMixin
 from sklearn.feature_extraction.text import TfidfVectorizer
 
+basicConfig(level=INFO)
+logger = getLogger(__name__)
+
 HAS_OPTIONAL_DEPENDENCIES = False
 
 try:
@@ -26,11 +30,10 @@ try:
     if HAS_OPTIONAL_DEPENDENCIES:
         nlp = spacy.load("en_core_web_sm")
 except OSError:
-    msg = (
-        "Spacy model is missing, install it with: "
-        f"{sys.executable} -m spacy download en_core_web_sm"
+    logger.error(
+        "Spacy model is missing, install it with: %s -m spacy download en_core_web_sm",
+        sys.executable,
     )
-    print(msg, file=sys.stderr)
 
 OPT_MSG_MISSING = (
     "Optional dependencies are missing, install them with: pip install bugbug[nlp]\n"
@@ -48,7 +51,6 @@ def spacy_token_lemmatizer(text):
 
 class SpacyVectorizer(TfidfVectorizer):
     def __init__(self, *args, **kwargs):
-
         # Detect when the Spacy optional dependency is missing
         if not HAS_OPTIONAL_DEPENDENCIES:
             raise NotImplementedError(OPT_MSG_MISSING)
