@@ -10,6 +10,7 @@ import itertools
 import json
 import logging
 import math
+import multiprocessing as mp
 import os
 import pickle
 import re
@@ -19,21 +20,11 @@ import sys
 import threading
 from datetime import datetime
 from functools import lru_cache
-from typing import (
-    Collection,
-    Dict,
-    Iterable,
-    Iterator,
-    List,
-    NewType,
-    Optional,
-    Set,
-    Tuple,
-    Union,
-)
+from typing import Collection, Iterable, Iterator, NewType, Set, Union
 
 import hglib
 import lmdb
+import requests
 import rs_parsepatch
 import tenacity
 from tqdm import tqdm
@@ -45,7 +36,7 @@ logger = logging.getLogger(__name__)
 
 CommitDict = NewType("CommitDict", dict)
 
-code_analysis_server: Optional[rust_code_analysis_server.RustCodeAnalysisServer] = None
+code_analysis_server: rust_code_analysis_server.RustCodeAnalysisServer | None = None
 
 hg_servers = list()
 hg_servers_lock = threading.Lock()
@@ -56,7 +47,7 @@ COMMIT_EXPERIENCES_DB = "commit_experiences.lmdb.tar.zst"
 db.register(
     COMMITS_DB,
     "https://community-tc.services.mozilla.com/api/index/v1/task/project.bugbug.data_commits.latest/artifacts/public/commits.json.zst",
-    22,
+    24,
     [COMMIT_EXPERIENCES_DB],
 )
 
@@ -71,7 +62,7 @@ EXPERIENCE_TIMESPAN_TEXT = f"{EXPERIENCE_TIMESPAN}_days"
 
 SOURCE_CODE_TYPES_TO_EXT = {
     "Assembly": [".asm", ".S"],
-    "Javascript": [".js", ".jsm", ".sjs"],
+    "Javascript": [".js", ".jsm", ".sjs", ".mjs", ".jsx"],
     "C/C++": [".c", ".cpp", ".cc", ".cxx", ".h", ".hh", ".hpp", ".hxx"],
     "Objective-C/C++": [".mm", ".m"],
     "Java": [".java"],
@@ -133,6 +124,18 @@ METRIC_NAMES = [
     "halstead_n1",
     "halstead_N2",
     "halstead_n2",
+    "halstead_length",
+    "halstead_estimated_program_length",
+    "halstead_purity_ratio",
+    "halstead_vocabulary",
+    "halstead_volume",
+    "halstead_difficulty",
+    "halstead_level",
+    "halstead_effort",
+    "halstead_time",
+    "halstead_bugs",
+    "functions",
+    "closures",
     "sloc",
     "ploc",
     "lloc",
@@ -171,11 +174,11 @@ class Commit:
         author: str,
         desc: str,
         pushdate: datetime,
-        bug_id: Optional[int],
-        backsout: List[str],
+        bug_id: int | None,
+        backsout: list[str],
         backedoutby: str,
         author_email: str,
-        reviewers: List[str],
+        reviewers: list[str],
         ignored: bool = False,
     ) -> None:
         self.node = node
@@ -195,7 +198,7 @@ class Commit:
         self.other_deleted = 0
         self.test_deleted = 0
         self.types: Set[str] = set()
-        self.functions: Dict[str, List[dict]] = {}
+        self.functions: dict[str, list[dict]] = {}
         self.seniority_author = 0.0
         self.total_source_code_file_size = 0
         self.average_source_code_file_size = 0.0
@@ -310,7 +313,7 @@ def get_commits(
     )
 
 
-def get_revision_id(commit: CommitDict) -> Optional[int]:
+def get_revision_id(commit: CommitDict) -> int | None:
     match = PHABRICATOR_REVISION_REGEX.search(commit["desc"])
     if not match:
         return None
@@ -357,7 +360,7 @@ def hg_modified_files(hg, commit):
         template=template,
         no_merges=True,
         rev=commit.node.encode("ascii"),
-        branch="tip",
+        branch="default",
     )
     x = hg.rawcommand(args)
     files_str, file_copies_str = x.split(b"\x00")[:-1]
@@ -395,7 +398,7 @@ def get_functions_from_metrics(metrics_space):
 
 def get_touched_functions(
     metrics_space: dict, deleted_lines: Iterable[int], added_lines: Iterable[int]
-) -> List[dict]:
+) -> list[dict]:
     touched_functions_indexes = set()
 
     functions = get_functions_from_metrics(metrics_space)
@@ -460,6 +463,49 @@ def get_summary_metrics(obj, metrics_space):
         obj["halstead_N2_max"] = max(obj["halstead_N2_max"], metrics["halstead"]["N2"])
         obj["halstead_n1_max"] = max(obj["halstead_n1_max"], metrics["halstead"]["n1"])
         obj["halstead_N1_max"] = max(obj["halstead_N1_max"], metrics["halstead"]["N1"])
+        if metrics["halstead"]["length"] is not None:
+            obj["halstead_length_max"] = max(
+                obj["halstead_length_max"], metrics["halstead"]["length"]
+            )
+        if metrics["halstead"]["estimated_program_length"] is not None:
+            obj["halstead_estimated_program_length_max"] = max(
+                obj["halstead_estimated_program_length_max"],
+                metrics["halstead"]["estimated_program_length"],
+            )
+        if metrics["halstead"]["purity_ratio"] is not None:
+            obj["halstead_purity_ratio_max"] = max(
+                obj["halstead_purity_ratio_max"], metrics["halstead"]["purity_ratio"]
+            )
+        if metrics["halstead"]["vocabulary"] is not None:
+            obj["halstead_vocabulary_max"] = max(
+                obj["halstead_vocabulary_max"], metrics["halstead"]["vocabulary"]
+            )
+        if metrics["halstead"]["volume"] is not None:
+            obj["halstead_volume_max"] = max(
+                obj["halstead_volume_max"], metrics["halstead"]["volume"]
+            )
+        if metrics["halstead"]["difficulty"] is not None:
+            obj["halstead_difficulty_max"] = max(
+                obj["halstead_difficulty_max"], metrics["halstead"]["difficulty"]
+            )
+        if metrics["halstead"]["level"] is not None:
+            obj["halstead_level_max"] = max(
+                obj["halstead_level_max"], metrics["halstead"]["level"]
+            )
+        if metrics["halstead"]["effort"] is not None:
+            obj["halstead_effort_max"] = max(
+                obj["halstead_effort_max"], metrics["halstead"]["effort"]
+            )
+        if metrics["halstead"]["time"] is not None:
+            obj["halstead_time_max"] = max(
+                obj["halstead_time_max"], metrics["halstead"]["time"]
+            )
+        if metrics["halstead"]["bugs"] is not None:
+            obj["halstead_bugs_max"] = max(
+                obj["halstead_bugs_max"], metrics["halstead"]["bugs"]
+            )
+        obj["functions_max"] = max(obj["functions_max"], metrics["nom"]["functions"])
+        obj["closures_max"] = max(obj["closures_max"], metrics["nom"]["closures"])
         obj["sloc_max"] = max(obj["sloc_max"], metrics["loc"]["sloc"])
         obj["ploc_max"] = max(obj["ploc_max"], metrics["loc"]["ploc"])
         obj["lloc_max"] = max(obj["lloc_max"], metrics["loc"]["lloc"])
@@ -485,6 +531,50 @@ def get_summary_metrics(obj, metrics_space):
         obj["halstead_N2_min"] = min(obj["halstead_N2_min"], metrics["halstead"]["N2"])
         obj["halstead_n1_min"] = min(obj["halstead_n1_min"], metrics["halstead"]["n1"])
         obj["halstead_N1_min"] = min(obj["halstead_N1_min"], metrics["halstead"]["N1"])
+
+        if metrics["halstead"]["length"] is not None:
+            obj["halstead_length_min"] = min(
+                obj["halstead_length_min"], metrics["halstead"]["length"]
+            )
+        if metrics["halstead"]["estimated_program_length"] is not None:
+            obj["halstead_estimated_program_length_min"] = min(
+                obj["halstead_estimated_program_length_min"],
+                metrics["halstead"]["estimated_program_length"],
+            )
+        if metrics["halstead"]["purity_ratio"] is not None:
+            obj["halstead_purity_ratio_min"] = min(
+                obj["halstead_purity_ratio_min"], metrics["halstead"]["purity_ratio"]
+            )
+        if metrics["halstead"]["vocabulary"] is not None:
+            obj["halstead_vocabulary_min"] = min(
+                obj["halstead_vocabulary_min"], metrics["halstead"]["vocabulary"]
+            )
+        if metrics["halstead"]["volume"] is not None:
+            obj["halstead_volume_min"] = min(
+                obj["halstead_volume_min"], metrics["halstead"]["volume"]
+            )
+        if metrics["halstead"]["difficulty"] is not None:
+            obj["halstead_difficulty_min"] = min(
+                obj["halstead_difficulty_min"], metrics["halstead"]["difficulty"]
+            )
+        if metrics["halstead"]["level"] is not None:
+            obj["halstead_level_min"] = min(
+                obj["halstead_level_min"], metrics["halstead"]["level"]
+            )
+        if metrics["halstead"]["effort"] is not None:
+            obj["halstead_effort_min"] = min(
+                obj["halstead_effort_min"], metrics["halstead"]["effort"]
+            )
+        if metrics["halstead"]["time"] is not None:
+            obj["halstead_time_min"] = min(
+                obj["halstead_time_min"], metrics["halstead"]["time"]
+            )
+        if metrics["halstead"]["bugs"] is not None:
+            obj["halstead_bugs_min"] = min(
+                obj["halstead_bugs_min"], metrics["halstead"]["bugs"]
+            )
+        obj["functions_min"] = min(obj["functions_min"], metrics["nom"]["functions"])
+        obj["closures_min"] = min(obj["closures_min"], metrics["nom"]["closures"])
         obj["sloc_min"] = min(obj["sloc_min"], metrics["loc"]["sloc"])
         obj["ploc_min"] = min(obj["ploc_min"], metrics["loc"]["ploc"])
         obj["lloc_min"] = min(obj["lloc_min"], metrics["loc"]["lloc"])
@@ -523,6 +613,30 @@ def get_space_metrics(
     obj["halstead_N2_total"] += metrics["halstead"]["N2"]
     obj["halstead_n1_total"] += metrics["halstead"]["n1"]
     obj["halstead_N1_total"] += metrics["halstead"]["N1"]
+    if metrics["halstead"]["length"] is not None:
+        obj["halstead_length_total"] += metrics["halstead"]["length"]
+    if metrics["halstead"]["estimated_program_length"] is not None:
+        obj["halstead_estimated_program_length_total"] += metrics["halstead"][
+            "estimated_program_length"
+        ]
+    if metrics["halstead"]["purity_ratio"] is not None:
+        obj["halstead_purity_ratio_total"] += metrics["halstead"]["purity_ratio"]
+    if metrics["halstead"]["vocabulary"] is not None:
+        obj["halstead_vocabulary_total"] += metrics["halstead"]["vocabulary"]
+    if metrics["halstead"]["volume"] is not None:
+        obj["halstead_volume_total"] += metrics["halstead"]["volume"]
+    if metrics["halstead"]["difficulty"] is not None:
+        obj["halstead_difficulty_total"] += metrics["halstead"]["difficulty"]
+    if metrics["halstead"]["level"] is not None:
+        obj["halstead_level_total"] += metrics["halstead"]["level"]
+    if metrics["halstead"]["effort"] is not None:
+        obj["halstead_effort_total"] += metrics["halstead"]["effort"]
+    if metrics["halstead"]["time"] is not None:
+        obj["halstead_time_total"] += metrics["halstead"]["time"]
+    if metrics["halstead"]["bugs"] is not None:
+        obj["halstead_bugs_total"] += metrics["halstead"]["bugs"]
+    obj["functions_total"] += metrics["nom"]["functions"]
+    obj["closures_total"] += metrics["nom"]["closures"]
     obj["sloc_total"] += metrics["loc"]["sloc"]
     obj["ploc_total"] += metrics["loc"]["ploc"]
     obj["lloc_total"] += metrics["loc"]["lloc"]
@@ -547,8 +661,8 @@ def get_space_metrics(
 def set_commit_metrics(
     commit: Commit,
     path: str,
-    deleted_lines: List[int],
-    added_lines: List[int],
+    deleted_lines: list[int],
+    added_lines: list[int],
     before_metrics: dict,
     after_metrics: dict,
 ) -> None:
@@ -757,7 +871,9 @@ def _transform(commit):
     return transform(HG, REPO_DIR, commit)
 
 
-def hg_log(hg: hglib.client, revs: List[bytes]) -> Tuple[Commit, ...]:
+def hg_log(
+    hg: hglib.client, revs: list[bytes], branch: str | None = "default"
+) -> tuple[Commit, ...]:
     if len(revs) == 0:
         return tuple()
 
@@ -768,7 +884,7 @@ def hg_log(hg: hglib.client, revs: List[bytes]) -> Tuple[Commit, ...]:
         template=template,
         no_merges=True,
         rev=revs,
-        branch="tip",
+        branch=branch,
     )
     x = hg.rawcommand(args)
     out = x.split(b"\x00")[:-1]
@@ -843,18 +959,18 @@ def hg_log(hg: hglib.client, revs: List[bytes]) -> Tuple[Commit, ...]:
     return tuple(commits)
 
 
-def _hg_log(revs: List[bytes]) -> Tuple[Commit, ...]:
-    return hg_log(thread_local.hg, revs)
+def _hg_log(revs: list[bytes], branch: str = "default") -> tuple[Commit, ...]:
+    return hg_log(thread_local.hg, revs, branch)
 
 
-def get_revs(hg, rev_start=0, rev_end="tip"):
-    logger.info(f"Getting revs from {rev_start} to {rev_end}...")
+def get_revs(hg, rev_start=0, rev_end="default"):
+    logger.info("Getting revs from %s to %s...", rev_start, rev_end)
 
     args = hglib.util.cmdbuilder(
         b"log",
         template="{node}\n",
         no_merges=True,
-        branch="tip",
+        branch="default",
         rev=f"{rev_start}:{rev_end}",
     )
     x = hg.rawcommand(args)
@@ -906,7 +1022,7 @@ class Experiences:
 def calculate_experiences(
     commits: Collection[Commit], first_pushdate: datetime, save: bool = True
 ) -> None:
-    logger.info(f"Analyzing seniorities from {len(commits)} commits...")
+    logger.info("Analyzing seniorities from %d commits...", len(commits))
 
     experiences = Experiences(save)
 
@@ -919,7 +1035,7 @@ def calculate_experiences(
             time_lapse = commit.pushdate - experiences[key]
             commit.seniority_author = time_lapse.total_seconds()
 
-    logger.info(f"Analyzing experiences from {len(commits)} commits...")
+    logger.info("Analyzing experiences from %d commits...", len(commits))
 
     # Note: In the case of files, directories, components, we can't just use the sum of previous commits, as we could end
     # up overcounting them. For example, consider a commit A which modifies "dir1" and "dir2", a commit B which modifies
@@ -930,7 +1046,7 @@ def calculate_experiences(
         return f"{exp_type}${commit_type}${item}"
 
     def get_experience(
-        exp_type: str, commit_type: str, item: str, day: int, default: Union[int, Tuple]
+        exp_type: str, commit_type: str, item: str, day: int, default: Union[int, tuple]
     ) -> utils.ExpQueue:
         key = get_key(exp_type, commit_type, item)
         try:
@@ -1091,7 +1207,8 @@ def set_commits_to_ignore(
     # 'ignore-this-changeset' in their description (mostly consisting of very
     # large and not meaningful formatting changes).
     ignore_revs_content = hg.cat(
-        [os.path.join(repo_dir, ".hg-annotate-ignore-revs").encode("ascii")], rev=b"-1"
+        [os.path.join(repo_dir, ".hg-annotate-ignore-revs").encode("ascii")],
+        rev=b"default",
     ).decode("utf-8")
     ignore_revs = set(line[:40] for line in ignore_revs_content.splitlines())
 
@@ -1104,19 +1221,22 @@ def set_commits_to_ignore(
 def download_coverage_mapping() -> None:
     commit_to_coverage = get_coverage_mapping(False)
 
-    utils.download_check_etag(
-        "https://firefox-ci-tc.services.mozilla.com/api/index/v1/task/project.relman.code-coverage.production.cron.latest/artifacts/public/commit_coverage.json.zst",
-        "data/coverage_mapping.json.zst",
-    )
+    try:
+        utils.download_check_etag(
+            "https://firefox-ci-tc.services.mozilla.com/api/index/v1/task/project.relman.code-coverage.production.cron.latest/artifacts/public/commit_coverage.json.zst",
+            "data/coverage_mapping.json.zst",
+        )
 
-    zstd_decompress("data/coverage_mapping.json")
-    assert os.path.exists("data/coverage_mapping.json")
+        zstd_decompress("data/coverage_mapping.json")
+        assert os.path.exists("data/coverage_mapping.json")
 
-    with open("data/coverage_mapping.json", "r") as f:
-        data = json.load(f)
+        with open("data/coverage_mapping.json", "r") as f:
+            data = json.load(f)
 
-    for commit_hash, commit_stats in data.items():
-        commit_to_coverage[commit_hash.encode("utf-8")] = pickle.dumps(commit_stats)
+        for commit_hash, commit_stats in data.items():
+            commit_to_coverage[commit_hash.encode("utf-8")] = pickle.dumps(commit_stats)
+    except requests.exceptions.HTTPError as e:
+        logger.error("Failure downloading commit->coverage mapping %s", e)
 
     close_coverage_mapping()
 
@@ -1187,7 +1307,9 @@ def close_component_mapping():
     path_to_component = None
 
 
-def hg_log_multi(repo_dir: str, revs: List[bytes]) -> Tuple[Commit, ...]:
+def hg_log_multi(
+    repo_dir: str, revs: list[bytes], branch: str | None = "default"
+) -> tuple[Commit, ...]:
     if len(revs) == 0:
         return tuple()
 
@@ -1200,7 +1322,7 @@ def hg_log_multi(repo_dir: str, revs: List[bytes]) -> Tuple[Commit, ...]:
     with concurrent.futures.ThreadPoolExecutor(
         initializer=_init_thread, initargs=(repo_dir,), max_workers=threads_num
     ) as executor:
-        commits_iter = executor.map(_hg_log, revs_groups)
+        commits_iter = executor.map(_hg_log, revs_groups, [branch] * len(revs_groups))
         commits_iter = tqdm(commits_iter, total=len(revs_groups))
         commits = tuple(itertools.chain.from_iterable(commits_iter))
 
@@ -1214,19 +1336,24 @@ def hg_log_multi(repo_dir: str, revs: List[bytes]) -> Tuple[Commit, ...]:
 @lru_cache(maxsize=None)
 def get_first_pushdate(repo_dir):
     with hglib.open(repo_dir) as hg:
-        return hg_log(hg, [b"0"])[0].pushdate
+        commits = hg_log(hg, [b"0"])
+        assert len(commits) == 1, (
+            f"There should be exactly one commit corresponding to revision 0: {commits}"
+        )
+        return commits[0].pushdate
 
 
 def download_commits(
     repo_dir: str,
-    rev_start: str = None,
-    revs: List[bytes] = None,
+    rev_start: str | None = None,
+    revs: list[bytes] | None = None,
+    branch: str | None = "default",
     save: bool = True,
     use_single_process: bool = False,
     include_no_bug: bool = False,
     include_backouts: bool = False,
     include_ignored: bool = False,
-) -> Tuple[CommitDict, ...]:
+) -> tuple[CommitDict, ...]:
     assert revs is not None or rev_start is not None
 
     with hglib.open(repo_dir) as hg:
@@ -1247,19 +1374,19 @@ def download_commits(
 
         first_pushdate = get_first_pushdate(repo_dir)
 
-        logger.info(f"Mining {len(revs)} commits...")
+        logger.info("Mining %d commits...", len(revs))
 
         if not use_single_process:
-            logger.info(f"Using {os.cpu_count()} processes...")
-            commits = hg_log_multi(repo_dir, revs)
+            logger.info("Using %d processes...", os.cpu_count())
+            commits = hg_log_multi(repo_dir, revs, branch)
         else:
-            commits = hg_log(hg, revs)
+            commits = hg_log(hg, revs, branch)
 
         set_commits_to_ignore(hg, repo_dir, commits)
 
         commits_num = len(commits)
 
-        logger.info(f"Mining {commits_num} patches...")
+        logger.info("Mining %d patches...", commits_num)
 
         global code_analysis_server
 
@@ -1267,7 +1394,10 @@ def download_commits(
             code_analysis_server = rust_code_analysis_server.RustCodeAnalysisServer()
 
             with concurrent.futures.ProcessPoolExecutor(
-                initializer=_init_process, initargs=(repo_dir,)
+                initializer=_init_process,
+                initargs=(repo_dir,),
+                # Fixing https://github.com/mozilla/bugbug/issues/3131
+                mp_context=mp.get_context("fork"),
             ) as executor:
                 commits_iter = executor.map(_transform, commits, chunksize=64)
                 commits_iter = tqdm(commits_iter, total=commits_num)
@@ -1277,7 +1407,7 @@ def download_commits(
 
             get_component_mapping()
 
-            commits = tuple(transform(hg, repo_dir, c) for c in commits)
+            commits = tuple(transform(hg, repo_dir, c) for c in tqdm(commits))
 
             close_component_mapping()
 
@@ -1364,9 +1494,9 @@ def clone(
 
         # Pull, to make sure the pushlog is generated.
         with hglib.open(repo_dir) as hg:
-            logger.info(f"Pulling {repo_dir}")
+            logger.info("Pulling %s", repo_dir)
             hg.pull(update=update)
-            logger.info(f"{repo_dir} pulled")
+            logger.info("%s pulled", repo_dir)
 
         return
     except hglib.error.ServerError as e:
@@ -1380,16 +1510,16 @@ def clone(
         purge=True,
         sharebase=repo_dir + "-shared",
         networkattempts=7,
-        branch=b"tip",
+        branch=b"default",
         noupdate=not update,
     )
     subprocess.run(cmd, check=True)
 
-    logger.info(f"{repo_dir} cloned")
+    logger.info("%s cloned", repo_dir)
 
 
 def pull(repo_dir: str, branch: str, revision: str) -> None:
-    """Pull a revision from a branch of a remote repository into a local repository"""
+    """Pull a revision from a branch of a remote repository into a local repository."""
 
     @tenacity.retry(
         stop=tenacity.stop_after_attempt(2),
@@ -1415,7 +1545,7 @@ def pull(repo_dir: str, branch: str, revision: str) -> None:
             raise
 
         if p.returncode != 0:
-            raise Exception(
+            raise RuntimeError(
                 f"Error {p.returncode} when pulling {revision} from {branch}"
             )
 
