@@ -3,7 +3,6 @@
 import argparse
 from datetime import datetime
 from logging import getLogger
-from typing import List
 
 import dateutil.parser
 from dateutil.relativedelta import relativedelta
@@ -15,32 +14,49 @@ logger = getLogger(__name__)
 
 
 class Retriever(object):
-    def retrieve_bugs(self, limit: int = None) -> None:
+    def retrieve_bugs(self, limit: int | None = None) -> None:
         bugzilla.set_token(get_secret("BUGZILLA_TOKEN"))
 
+        last_modified = None
         db.download(bugzilla.BUGS_DB)
 
         # Get IDs of bugs changed since last run.
-        last_modified = db.last_modified(bugzilla.BUGS_DB)
-        logger.info(
-            f"Retrieving IDs of bugs modified since the last run on {last_modified}"
-        )
-        changed_ids = set(
-            bugzilla.get_ids(
-                {"f1": "delta_ts", "o1": "greaterthaneq", "v1": last_modified.date()}
-            )
-        )
-        logger.info(f"Retrieved {len(changed_ids)} IDs.")
+        try:
+            last_modified = db.last_modified(bugzilla.BUGS_DB)
+        except db.LastModifiedNotAvailable:
+            pass
 
-        all_components = bugzilla.get_product_component_count(9999)
+        if last_modified is not None:
+            logger.info(
+                "Retrieving IDs of bugs modified since the last run on %s",
+                last_modified,
+            )
+            changed_ids = set(
+                bugzilla.get_ids(
+                    {
+                        "f1": "delta_ts",
+                        "o1": "greaterthaneq",
+                        "v1": last_modified.date(),
+                    }
+                )
+            )
+        else:
+            changed_ids = set()
+
+        logger.info("Retrieved %d IDs.", len(changed_ids))
+
+        all_components = set(bugzilla.fetch_components_list())
 
         deleted_component_ids = set(
             bug["id"]
-            for bug in bugzilla.get_bugs()
-            if "{}::{}".format(bug["product"], bug["component"]) not in all_components
+            for bug in bugzilla.get_bugs(
+                include_invalid=True,
+                include_additional_products=bugzilla.ADDITIONAL_PRODUCTS,
+            )
+            if (bug["product"], bug["component"]) not in all_components
         )
         logger.info(
-            f"{len(deleted_component_ids)} bugs belonging to deleted components"
+            "%d bugs belonging to deleted components", len(deleted_component_ids)
         )
         changed_ids |= deleted_component_ids
 
@@ -48,17 +64,17 @@ class Retriever(object):
         two_years_and_six_months_ago = datetime.utcnow() - relativedelta(
             years=2, months=6
         )
-        logger.info(f"Retrieving bug IDs since {two_years_and_six_months_ago}")
+        logger.info("Retrieving bug IDs since %s", two_years_and_six_months_ago)
         timespan_ids = bugzilla.get_ids_between(two_years_and_six_months_ago)
         if limit:
             timespan_ids = timespan_ids[-limit:]
-        logger.info(f"Retrieved {len(timespan_ids)} IDs.")
+        logger.info("Retrieved %d IDs.", len(timespan_ids))
 
         # Get IDs of labelled bugs.
         labelled_bug_ids = labels.get_all_bug_ids()
         if limit:
             labelled_bug_ids = labelled_bug_ids[-limit:]
-        logger.info(f"{len(labelled_bug_ids)} labelled bugs to download.")
+        logger.info("%d labelled bugs to download.", len(labelled_bug_ids))
 
         # Get the commits DB, as we need it to get the bug IDs linked to recent commits.
         # XXX: Temporarily avoid downloading the commits DB when a limit is set, to avoid the integration test fail when the commits DB is bumped.
@@ -77,16 +93,18 @@ class Retriever(object):
         )
         if limit:
             commit_bug_ids = commit_bug_ids[-limit:]
-        logger.info(f"{len(commit_bug_ids)} bugs linked to commits to download.")
+        logger.info("%d bugs linked to commits to download.", len(commit_bug_ids))
 
         # Get IDs of bugs which are regressions, bugs which caused regressions (useful for the regressor model),
         # and blocked bugs.
-        regression_related_ids: List[int] = list(
+        regression_related_ids: list[int] = list(
             set(
                 sum(
                     (
                         bug["regressed_by"] + bug["regressions"] + bug["blocks"]
-                        for bug in bugzilla.get_bugs()
+                        for bug in bugzilla.get_bugs(
+                            include_additional_products=bugzilla.ADDITIONAL_PRODUCTS
+                        )
                     ),
                     [],
                 )
@@ -95,7 +113,8 @@ class Retriever(object):
         if limit:
             regression_related_ids = regression_related_ids[-limit:]
         logger.info(
-            f"{len(regression_related_ids)} bugs which caused regressions fixed by commits."
+            "%d bugs which caused regressions fixed by commits.",
+            len(regression_related_ids),
         )
 
         # Get IDs of bugs linked to intermittent failures.
@@ -104,10 +123,11 @@ class Retriever(object):
             for item in test_scheduling.get_failure_bugs(
                 two_years_and_six_months_ago, datetime.utcnow()
             )
+            if item["bug_id"] is not None
         ]
         if limit:
             test_failure_bug_ids = test_failure_bug_ids[-limit:]
-        logger.info(f"{len(test_failure_bug_ids)} bugs about test failures.")
+        logger.info("%d bugs about test failures.", len(test_failure_bug_ids))
 
         all_ids = (
             timespan_ids
@@ -140,7 +160,8 @@ class Retriever(object):
                 )
             )
             logger.info(
-                f"{len(regression_related_ids)} bugs which caused regressions fixed by commits."
+                "%d bugs which caused regressions fixed by commits.",
+                len(regression_related_ids),
             )
             if limit:
                 regression_related_ids = regression_related_ids[-limit:]
@@ -152,7 +173,10 @@ class Retriever(object):
             new_bugs = bugzilla.download_bugs(regression_related_ids)
 
         # Try to re-download inconsistent bugs, up to twice.
-        inconsistent_bugs = bugzilla.get_bugs(include_invalid=True)
+        inconsistent_bugs = bugzilla.get_bugs(
+            include_invalid=True,
+            include_additional_products=bugzilla.ADDITIONAL_PRODUCTS,
+        )
         for i in range(2):
             # We look for inconsistencies in all bugs first, then, on following passes,
             # we only look for inconsistencies in bugs that were found to be inconsistent in the first pass
@@ -163,23 +187,65 @@ class Retriever(object):
                 break
 
             logger.info(
-                f"Re-downloading {len(inconsistent_bug_ids)} bugs, as they were inconsistent"
+                "Re-downloading %d bugs, as they were inconsistent",
+                len(inconsistent_bug_ids),
             )
             bugzilla.delete_bugs(lambda bug: bug["id"] in inconsistent_bug_ids)
             bugzilla.download_bugs(inconsistent_bug_ids)
 
-        # TODO: Figure out why.
-        missing_history_bug_ids = {
-            bug["id"] for bug in bugzilla.get_bugs() if "history" not in bug
-        }
-        bugzilla.delete_bugs(lambda bug: bug["id"] in missing_history_bug_ids)
-        logger.info(
-            f"Deleted {len(missing_history_bug_ids)} bugs as we couldn't retrieve their history"
-        )
+        # TODO: Figure out why we have missing fields in the first place.
+        handle_missing_fields(["history", "comments"])
 
         zstd_compress(bugzilla.BUGS_DB)
 
 
+def handle_missing_fields(
+    fields: list[str], trial_number: int = 1, max_tries: int = 2
+) -> int:
+    """Handle bugs that are missing a mandatory field.
+
+    This function will try to re-download the bugs that are missing any of the
+    given fields and delete them if they are still missing a field after the
+    maximum number of tries.
+
+    Args:
+        fields: The list of field names to check for.
+        trial_number: The current try number. It should be 1 on the first call.
+        max_tries: The maximum number of tries to re-download the bugs.
+
+    Returns:
+        Number of bugs that were deleted.
+    """
+    missing_field_bug_ids = {
+        bug["id"]
+        for bug in db.read(bugzilla.BUGS_DB)
+        if any(field not in bug for field in fields)
+    }
+
+    if not missing_field_bug_ids:
+        return 0
+
+    bugzilla.delete_bugs(lambda bug: bug["id"] in missing_field_bug_ids)
+
+    if trial_number <= max_tries:
+        logger.info(
+            "Re-downloading %d bugs, as they were missing the fields (re-trial %d of %d)",
+            len(missing_field_bug_ids),
+            trial_number,
+            max_tries,
+        )
+        bugzilla.download_bugs(missing_field_bug_ids)
+
+        return handle_missing_fields(fields, trial_number + 1, max_tries)
+
+    logger.info(
+        "Deleted %d bugs as we couldn't retrieve their missing fields",
+        len(missing_field_bug_ids),
+    )
+
+    return len(missing_field_bug_ids)
+
+
 def main() -> None:
     description = "Retrieve and extract the information from Bugzilla instance"
     parser = argparse.ArgumentParser(description=description)
