@@ -9,6 +9,10 @@ import json
 import os
 import sys
 from datetime import date, datetime, timedelta
+from logging import INFO, basicConfig, getLogger
+
+basicConfig(level=INFO)
+logger = getLogger(__name__)
 
 # Inject project path
 sys.path.append("../")
@@ -41,18 +45,16 @@ def fetch_untriaged(args):
 
 def run_untriaged(untriaged_bugs):
     from bugbug.models.component import ComponentModel
-    from bugbug.models.component_nn import ComponentNNModel
 
     models = [
         (ComponentModel, "../componentmodel"),
-        (ComponentNNModel, "../componentnnmodel"),
     ]
-    for (model_class, model_file_name) in models:
+    for model_class, model_file_name in models:
         rows = []
         model = model_class.load(model_file_name)
         for bug in untriaged_bugs:
             p = model.classify(bug, probabilities=True)
-            url = f'https://bugzilla.mozilla.org/show_bug.cgi?id={bug["id"]}'
+            url = f"https://bugzilla.mozilla.org/show_bug.cgi?id={bug['id']}"
 
             classifiable = p[p >= 0.7].size >= 1
             classification = ""
@@ -61,16 +63,16 @@ def run_untriaged(untriaged_bugs):
                 bug["product"], bug["component"]
             )
             if not expected_component:
-                print("Skipping bug: {}".format(bug["id"]))
+                logger.info("Skipping bug: %s", bug["id"])
                 continue
 
             if classifiable:
-                print("Classifying bug with ID: {}".format(bug["id"]))
+                logger.info("Classifying bug with ID: %s", bug["id"])
                 classification = model.classify(bug)[0]
-                print("Classified bug as: {}".format(classification))
+                logger.info("Classified bug as: %s", classification)
 
             else:
-                print("Not classifiable bug: {}".format(bug["id"]))
+                logger.info("Not classifiable bug: %s", bug["id"])
 
             correct_prediction = expected_component == classification
             rows.append(
