@@ -19,11 +19,10 @@ logger = getLogger(__name__)
 def classify_issues(
     owner: str, repo: str, retrieve_events: bool, model_name: str, issue_number: int
 ) -> None:
-
     model_file_name = f"{model_name}model"
 
     if not os.path.exists(model_file_name):
-        logger.info(f"{model_file_name} does not exist. Downloading the model....")
+        logger.info("%s does not exist. Downloading the model....", model_file_name)
         try:
             download_model(model_name)
         except requests.HTTPError:
@@ -49,7 +48,7 @@ def classify_issues(
         issues = github.get_issues()
 
     for issue in issues:
-        print(f'{issue["url"]} - {issue["title"]} ')
+        logger.info("%s - %s ", issue["url"], issue["title"])
 
         if model.calculate_importance:
             probas, importance = model.classify(
@@ -68,7 +67,7 @@ def classify_issues(
             pred_class = model.le.inverse_transform([pred_index])[0]
         else:
             pred_class = "Positive" if pred_index == 1 else "Negative"
-        print(f"{pred_class} {probability}")
+        logger.info("%s %s", pred_class, probability)
         input()
 
 
