@@ -9,9 +9,7 @@ from logging import INFO, basicConfig, getLogger
 
 from bugbug import db
 from bugbug.models import MODELS, get_model_class
-from bugbug.utils import CustomJsonEncoder, zstd_compress
-
-MODELS_WITH_TYPE = ("component",)
+from bugbug.utils import CustomJsonEncoder, create_tar_zst, zstd_compress
 
 basicConfig(level=INFO)
 logger = getLogger(__name__)
@@ -22,15 +20,7 @@ class Trainer(object):
         # Download datasets that were built by bugbug_data.
         os.makedirs("data", exist_ok=True)
 
-        if args.classifier != "default":
-            assert (
-                args.model in MODELS_WITH_TYPE
-            ), f"{args.classifier} is not a valid classifier type for {args.model}"
-
-            model_name = f"{args.model}_{args.classifier}"
-        else:
-            model_name = args.model
-
+        model_name = args.model
         model_class = get_model_class(model_name)
         parameter_names = set(inspect.signature(model_class.__init__).parameters)
         parameters = {
@@ -47,7 +37,7 @@ class Trainer(object):
         else:
             logger.info("Skipping download of the databases")
 
-        logger.info(f"Training *{model_name}* model")
+        logger.info("Training *%s* model", model_name)
         metrics = model_obj.train(limit=args.limit)
 
         # Save the metrics as a file that can be uploaded as an artifact.
@@ -57,17 +47,17 @@ class Trainer(object):
 
         logger.info("Training done")
 
-        model_file_name = f"{model_name}model"
-        assert os.path.exists(model_file_name)
-        zstd_compress(model_file_name)
+        model_directory = f"{model_name}model"
+        assert os.path.exists(model_directory)
+        create_tar_zst(f"{model_directory}.tar.zst")
 
         logger.info("Model compressed")
 
         if model_obj.store_dataset:
-            assert os.path.exists(f"{model_file_name}_data_X")
-            zstd_compress(f"{model_file_name}_data_X")
-            assert os.path.exists(f"{model_file_name}_data_y")
-            zstd_compress(f"{model_file_name}_data_y")
+            assert os.path.exists(f"{model_name}model_data_X")
+            zstd_compress(f"{model_name}model_data_X")
+            assert os.path.exists(f"{model_name}model_data_y")
+            zstd_compress(f"{model_name}model_data_y")
 
 
 def parse_args(args):
@@ -97,14 +87,8 @@ def parse_args(args):
         help="Perform lemmatization (using spaCy)",
         action="store_true",
     )
-    parser.add_argument(
-        "--classifier",
-        help="Type of the classifier. Only used for component classification.",
-        choices=["default", "nn"],
-        default="default",
-    )
 
-    subparsers = main_parser.add_subparsers(title="model", dest="model")
+    subparsers = main_parser.add_subparsers(title="model", dest="model", required=True)
 
     for model_name in MODELS:
         subparser = subparsers.add_parser(
@@ -129,14 +113,16 @@ def parse_args(args):
                 parameter_type = type(parameter.default)
             assert parameter_type is not None
 
-            if parameter_type == bool:
+            if parameter_type is bool:
                 subparser.add_argument(
-                    f"--{parameter.name}"
-                    if parameter.default is False
-                    else f"--no-{parameter.name}",
-                    action="store_true"
-                    if parameter.default is False
-                    else "store_false",
+                    (
+                        f"--{parameter.name}"
+                        if parameter.default is False
+                        else f"--no-{parameter.name}"
+                    ),
+                    action=(
+                        "store_true" if parameter.default is False else "store_false"
+                    ),
                     dest=parameter.name,
                 )
             else:
